#ifndef LP_MAIN_WINDOW
#define LP_MAIN_WINDOW

#include "windows.h"

#include <QWidget>
#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QComboBox>
#include <QGroupBox>
#include <QListWidget>
#include <QMessageBox>
#include <QTreeWidget>
#include <QLineEdit>
#include <QGridLayout>
#include <QTreeWidgetItem>
#include <QTreeWidget>
#include <QButtonGroup>
#include <QMainWindow>
#include <QWizard>
#include <QWizardPage>

// #define USE_ZEROC_ICE

#ifdef USE_ZEROC_ICE
	#include "IceStormCommunication.h"
#endif


#include "Plot.h"
#include "GraphWindow.h"
#include "ThreeDWindow.h"
#include "SaveDataThread.h"
#include "StdRedirector.h"
#include "SensorGuiContainer.h"
#include "LpmsSensorManagerI.h"
#include "DiscoveryTree.h"
#include "LpmsSensorI.h"
#include "LpmsDefinitions.h"
#include "FieldMapContainer.h"
#include "SanAngelesWindow.h"

#include <string>
#include <iostream>
#include <vector>
#include <list>
using namespace std;

#include <boost/shared_ptr.hpp>
using namespace boost;

#define CALIBRATION_FILE "LpmsControlConfiguration.xml"

/*! \brief	The main window of LpmsControl containing GUI and sensor administration.
	This class manages the control window on the left hand side of the application screen,
	the graph / 3D window in the middle and the status display on the right. A few
	of the connection and status message callbacks are still located in MainWindow
	methods, but I plan to move them to some other place like LpmsContainer soon,
	also to make porting the program tp other IMU systems easier. */
class MainWindow : public QMainWindow
{
Q_OBJECT
	
public:
	/*! \brief	Initialization of the main application window. */
	MainWindow(QWidget *parent = 0);
	
	/*! \brief	Destroys the MainWindow. */
	~MainWindow();
	
	/*! \brief	Builds the graph and 3d window. Both are inserted into a layout,
				only one is shown at a time. */	
	QGroupBox *doGraph(void);
	
	/*! \brief	Builds the GUI elements for the status group box. */	
	QGroupBox *doStatus(void);
		
	/*! \brief	Builds the GUI elements for the LPMS control group box. */
	QGroupBox *doLpms(void);

	/*! \brief	Builds console window to channel console output to GUI. Disabled. */	
	QGroupBox *doConsole(void);		
	
	QWizardPage *maOrientationPage(char* ts, char* es);
		
public slots:	
	/*! \brief	Switches the center window from ThreeDWindow to GraphWindow. */	
	void selectGraphWindow(void);
	
	void selectGraph2Window(void);
	
	void selectGraph3Window(void);

	/*! \brief	Switches the center window from GraphWindow to ThreeDWindow. */
	void selectThreeDWindow(void);
	
	/*! \brief	Allows the user to terminate the application.
		If exit is confirmed, the application terminates. */	
	bool exitWindow(void);
	
	/*! \brief	Establishes a new sensor connection. */	
	void openSensor(void);
	
	/*! \brief	Closes the connection to the currently selected device. */	
	void closeSensor(void);

	/*! \brief	Starts or stops the sensor measurement depending on the isRunning 
				flag. */
	void startMeasurement(void);
	
	/*! \brief	Stops the measurement. 
		Used for force-stopping the measurement in case of sensor connection 
		interruption etc. */	
	void stopMeasurement(void);
	
	/*! \brief	Starts the accelerometer and gyroscope calibration of the selected sensor. */	
	void recalibrate(void);
	
	/*! \brief	Starta the recording of the sensor data. */	
	void recordData(void);
	
	/*! \brief	Updates the pointer to the currently selected LPMS object. 
		This is necessary when the user changes the sensor selection in the tree view, 
		a new sensor is added or a sensor is disconnected. */
	void updateCurrentLpms(QTreeWidgetItem *current = 0, QTreeWidgetItem *previous = 0);
	
	/*! \brief	Sets the zero offset matrix for the selected sensor. */	
	void zeroAngle(void);
	
	/*! \brief	Sets the zero offset matrix for the selected sensor. */	
	void zeroAngleSelected(void);
	
	/*! \brief	Starts the calibration of the magnetometer of the selected sensor. */	
	void calibrateMag(void);
	
	/*! \brief	Saves the calibration data. */	
	void saveCalibration(void);
	
	/*! \brief	Connects to OpenMAT server (actually a ICE IceBox server). */	
	void startServer(void);
	
	/*! \brief	Updates the variable holding the current sensor data transfer frame rate. */	
	void updateLpmsFps(int v, int lpmsId);
	
	/*! \brief	Updates the current sensor status shown in the status window and
				checks if there is new data from the sensor. */
	void timerUpdate(void);
	
	void discoveryTimerUpdate(void);
	
	/*! Starts uploading a new firmware file. */
	void uploadFirmware(void);

	/*! Starts uploading a new IAP file. */
	void uploadIap(void);

	/*! Initilizes application menu bar. */
	void doMenus(void);
	
	void uploadTimerUpdate(void);
	
	void toggleSelfTest(void);
	
	void measureLatency(void);
	
	void getFieldMap(void);
	
	void selectFieldMapWindow(void);
	
	void getVersionInfo(void);
	
	void closeEvent(QCloseEvent *event);
	
	void resetToFactory(void);
	
	void misalignmentCal(void);
	
	void maNewPage(int i);
	
	void maFinished(int i);
	
	QWizardPage *maFinishedPage(void);
	
	void calTimerUpdate(void);
	
	void startWaitBar(int t);

	void addRemoveDevices(void);
	
	void saveCalibrationData(void);

	void loadCalibrationData(void);
	
private:
#ifdef USE_ZEROC_ICE
	IceStormPublisher *isp;		
#endif
	QList<QTreeWidgetItem *> lpmsTreeItems;
	QTreeWidget *lpmsTree;	
	GraphWindow *graphWindow;
	ThreeDWindow *threeDWindow;
	FieldMapContainer *fieldMapWindow;
	SanAngelesWindow *sanAngeles;
	SaveDataThread *sd;
	SensorGuiContainer *currentLpms;
	list<SensorGuiContainer *> lpmsList;
	LpmsSensorManagerI *sm;
	QLabel *imuFpsLabel;
	QLabel *bluetoothStatusLabel;
	QLabel *lpmsStatusLabel;
	QLabel *xAccLabel;
	QLabel *yAccLabel;
	QLabel *zAccLabel;
	QLabel *xGyroLabel;
	QLabel *yGyroLabel;
	QLabel *zGyroLabel;
	QLabel *xMagLabel;
	QLabel *yMagLabel;
	QLabel *zMagLabel;
	QLabel *xAngleLabel;
	QLabel *yAngleLabel;
	QLabel *zAngleLabel;
	QLabel *xQuatLabel;
	QLabel *yQuatLabel;
	QLabel *zQuatLabel;
	QLabel *wQuatLabel;
	QLabel *frameCountLabel; 
	QLabel *activeLpmsLabel; 
	QLabel *magRangeLabel;
	QLabel *pressureLabel;
	QLabel *xLinAccLabel;
	QLabel *yLinAccLabel;
	QLabel *zLinAccLabel;
	QLineEdit *deviceAddressEdit;
	QLineEdit *ftdiDeviceEdit;
	QPushButton *recordDataButton;
	QPushButton *startButton;
	QPushButton *connectButton;
	QPushButton *calibrateMagButton;
	QPushButton *startServerButton;	
	QPushButton *graphWindowButton;
	QPushButton *graphWindow2Button;
	QPushButton *graphWindow3Button;
	QPushButton *threeDWindowButton;
	QPushButton *fieldMapWindowButton;
	bool isRunning;
	bool isConnecting;	
	bool calibratingMag;
	ConsoleWindow *consoleWindow;
	QSlider *accGainSl;
	QSlider *magGainSl;
	QSlider *accCovarSl;
	QSlider *magCovarSl;	
	QLineEdit *csvFileEdit;
	QRadioButton* gyrOnlyBtn;
    QRadioButton* gyrAccBtn;
    QRadioButton* accMagBtn;
    QRadioButton* gyrAccMagNsBtn;
    QRadioButton* gyrAccMagSwBtn;
	QRadioButton* gTEnableBtn;
	QRadioButton* gTDisableBtn;
	QComboBox* deviceTypeSelector;
	QComboBox *addressSelector;
	DiscoveryTree *deviceTree;
	QAction* startAction;
	QAction* saveAction;
	QAction* selfTestAction;
	int textUpdateCounter;
	QTimer* uploadTimer;
	QProgressDialog* uploadProgress;
	QWizard* maWizard;
	bool maCalibrationFinished;
	int calMaxTime;
	int calTime;
	QProgressDialog *calProgress;
	QTimer *calTimer;
	bool maIsCalibrating;
};

#endif