/****************************************************************************
**
** Copyright (C) 2011 LP-Research
** All rights reserved.
** Contact: LP-Research (klaus@lp-research.com)
**
** This file is part of the Open Motion Analysis Toolkit (OpenMAT).
**
** OpenMAT is free software: you can redistribute it and/or modify it under 
** the terms of the GNU General Public License as published by the Free 
** Software Foundation, either version 3 of the License, or (at your option) 
** any later version.
** 
** OpenMAT is distributed in the hope that it will be useful, but WITHOUT 
** ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
** FITNESS FOR A PARTICULAR PURPOSE. See the GNU \ General Public License 
** for more details.
** 
** You should have received a copy of the GNU General Public License along 
** with the OpenMAT library. If not, see <http://www.gnu.org/licenses/>.
**
****************************************************************************/

#include "SaveDataThread.h"

using namespace std;

boost::mutex m;

SaveDataThread::SaveDataThread(void)
{
	saveImuStarted = false;
}

SaveDataThread::~SaveDataThread(void)
{
	stopSaveImuData();
	
	boost::this_thread::sleep(boost::posix_time::milliseconds(50));	
}

bool SaveDataThread::startSaveImuData(string fn)
{
	std::list<LpmsSensorI*>::iterator i;

	imuDataStream.open(fn.c_str());

	if (imuDataStream.is_open()) {
		for (i = sensorList.begin(); i != sensorList.end(); ++i) {		
			if (i != sensorList.begin()) imuDataStream << ", ";
			imuDataStream << "SensorId, TimeStamp (s), FrameNumber, " << 
				"AccX (g), AccY (g), AccZ (g), " <<
				"GyroX (deg/s), GyroY (deg/s), GyroZ (deg/s), " <<
				"MagX (uT), MagY (uT), MagZ (uT), " <<
				"EulerX (deg), EulerY (deg), EulerZ (deg), " <<
				"QuatX, QuatY, QuatZ, QuatW";
		}
			
		imuDataStream << endl;

		saveImuStarted = true;		
		
		boost::thread t(&SaveDataThread::threadRun, this);
		t.detach();	

		cout << "[SaveDataThread] Writing LPMS data to " << fn.c_str() << endl;	
		
		return true;
	}
	
	cout << "[SaveDataThread] Failed to open " << fn.c_str() << endl;	
	
	return false;
}

bool SaveDataThread::isRecording(void)
{
	return saveImuStarted;
}

void SaveDataThread::stopSaveImuData(void)
{
	saveImuStarted = false;
	
	boost::this_thread::sleep(boost::posix_time::milliseconds(100));		
	
	if (imuDataStream.is_open()) imuDataStream.close();
}

void SaveDataThread::threadRun(void)
{
	MicroMeasure mm;
	std::list<LpmsSensorI*>::iterator i;	
	ImuData d;
	int currentTimeSlice;
	int dataSavePeriod;
		
	for (i = sensorList.begin(); i != sensorList.end(); ++i) 
	{	
		(*i)->resetTimestamp();
	}		

	boost::this_thread::sleep(boost::posix_time::milliseconds(20));		
	
	currentTimeSlice = 1;
	while (saveImuStarted == true) {		
		if (mm.measure() > DATA_SAVE_PERIOD) {
			mm.reset();
			
			if (currentTimeSlice == 200) {
				currentTimeSlice = 1;
			} else {
				++currentTimeSlice;
			}
						
			for (i = sensorList.begin(); i != sensorList.end(); i++) {			
				dataSavePeriod = getStreamFrequency(*i);
				if (currentTimeSlice % dataSavePeriod == 0)	{			
					// if (i != sensorList.begin()) imuDataStream << ", ";
					d = (*i)->getCurrentData();
					writeImuData(d);
				}
			}
			
			// imuDataStream << endl;
		}
		
		boost::this_thread::sleep(boost::posix_time::microseconds(500));	
	}
	
	if (imuDataStream.is_open()) imuDataStream.close();
}

long SaveDataThread::getStreamFrequency(LpmsSensorI *s)
{
	int i;
	int dataSavePeriod;
	
	s->getConfigurationPrm(PRM_SAMPLING_RATE, &i);	
	
	switch (i) {
	case SELECT_STREAM_FREQ_5HZ:
		dataSavePeriod = 200;
	break;		
	
	case SELECT_STREAM_FREQ_10HZ:
		dataSavePeriod = 100;
	break;	
	
	case SELECT_STREAM_FREQ_50HZ:		
		dataSavePeriod = 50;				
	break;
		
	case SELECT_STREAM_FREQ_100HZ:
		dataSavePeriod = 10;
	break;
		
	case SELECT_STREAM_FREQ_200HZ:
		dataSavePeriod = 5;
	break;
	
	case SELECT_STREAM_FREQ_500HZ:
		dataSavePeriod = 2;
	break;	
	
	default:
		dataSavePeriod = 2;
	break;
	}
	
	return dataSavePeriod;
}

void SaveDataThread::writeImuData(ImuData d)
{		
	if (imuDataStream.is_open()) {					
		imuDataStream << d.openMatId << ", " << d.timestamp << ", " << d.frameCount << ", " <<
			d.a[0] << ", " << d.a[1] << ", "  << d.a[2] << ", " <<
			d.g[0] << ", " << d.g[1] << ", " << d.g[2] << ", " <<
			d.b[0] << ", " << d.b[1] << ", " << d.b[2] << ", " <<
			d.r[0] << ", " << d.r[1] << ", " << d.r[2] << ", " <<
			d.q[0] << ", " << d.q[1] << ", " << d.q[2] << ", " << d.q[3] <<
			std::endl; 
	}
}

void SaveDataThread::addSensor(LpmsSensorI* sensor)
{
	sensorList.push_back(sensor);
}

void SaveDataThread::removeSensor(LpmsSensorI* sensor)
{
	sensorList.remove(sensor);
}